<?php defined('SYSPATH') || die('No direct script access.');

class Model_Faculties extends Model
{
    // todo: should generate an array of Model_Faculty
    public static function load() {
        $sql = 'CALL `GetFaculties`(); ';
        return DB::query(Database::SELECT, $sql)
            ->execute()->as_array();
    }

    public static function createFaculty($externalID, $name, $abbr) {
        $sql = 'SELECT `Faculty_Create`(:externalID, :name, :abbr) AS `ID`;';
        $query = DB::query(Database::SELECT, $sql)
            ->param(':externalID', $externalID)
            ->param(':name', $name)
            ->param(':abbr', $abbr);

        $id = $query->execute();

        if ($id->count() == 0)
            return NULL;

        return $id->get('ID');
    }

    public static function getDepartments($facultyID = 0) {
        $sql = 'CALL `Departments_Get`(:faculty)';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':faculty', $facultyID)
            ->execute();
        
        $list = [];
        foreach ($result as $row) {
            if (empty($row['Name']))
                $row['Name'] = 'Без кафедры';
            
            $list[] = $row;
        }
        
        return $list;
    }

    public static function getDepartmentIdByName($departmentName) {
        $sql = 'CALL `Department_GetIdByName`(:name)';
        $id = DB::query(Database::SELECT, $sql)
            ->param(':name', $departmentName)
            ->execute();

        if ($id->count() == 0)
            throw new InvalidArgumentException(Error::DEPARTMENT_NOT_FOUND);

        return $id->get('ID');
    }

    public static function getDepartmentIdByExternalID($departmentExternalID) {
        $sql = 'CALL `Department_GetIdByExternalID`(:depExternalID)';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':depExternalID', $departmentExternalID)
            ->execute();

        if ($result->count() == 0)
            throw new InvalidArgumentException(Error::DEPARTMENT_NOT_FOUND);

        return $result->get('ID');
    }

    public static function getIdByExternalID($facultyExternalID) {
        $sql = 'SELECT `Faculty_GetIdByExternalID`(:facultyExternalID) AS `ID`';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':facultyExternalID', $facultyExternalID)
            ->execute();

        $ID = (int)$result->get('ID');

        if ($ID == -1)
            throw new InvalidArgumentException(Error::DEPARTMENT_NOT_FOUND);

        return $ID;
    }

    public static function createDepartment($departmentName, $facultyID) {
        $sql = 'SELECT `Department_Create`(:name, :facultyID) AS `ID`;';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':name', $departmentName)
            ->param(':facultyID', $facultyID)
            ->execute();

        if ($result->count() == 0)
            return NULL;

        return $result->get('ID');
    }

    public static function createDepartmentWithExternalID($externalID, $name, $facultyExternalID) {
        $sql = 'SELECT `Department_Create_With_ExternalID`(:externalID, :name, :facultyExternalID) AS `ID`;';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':externalID', $externalID)
            ->param(':name', $name)
            ->param(':facultyExternalID', $facultyExternalID)
            ->execute();

        if ($result->count() == 0)
            return NULL;

        return $result->get('ID');
    }

    public static function getJobPositionIdByName($jobPositionName) {
        $sql = 'CALL `JobPosition_GetIdByName`(:name)';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':name', $jobPositionName)
            ->execute();

        if ($result->count() == 0)
            return NULL;

        return $result->get('ID');
    }

    public static function createJobPosition($jobPositionName) {
        $sql = 'SELECT `JobPosition_Create`(:name) AS `ID`;';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':name', $jobPositionName)
            ->execute();

        if ($result->count() == 0)
            return NULL;

        return $result->get('ID');
    }
}
